#include "mysql/mysql.h"		    /* For MYSQL APIs */
#include <string.h>
#include <stdio.h>

/*
 *	Displays query results
 */
int processQueryResult(MYSQL *pConn, MYSQL_RES *pRes)
{
	MYSQL_ROW 		aRow;
	unsigned int 	iCounter, iTotal;
	
	aRow = mysql_fetch_row (pRes);
	while (NULL != aRow)
	{
		/* Basic formatting for now */
		iTotal = mysql_num_fields (pRes);
		for (iCounter = 0; iCounter < iTotal; iCounter++)
		{
			/* Check if empty data first before displaying */
			if (NULL != aRow[iCounter])
			{
				printf ("%s\t", aRow[iCounter]);
			}
			else
			{
				printf ("NULL\t");
			}
		} /* For each field/column */
		printf ("\n");
		aRow = mysql_fetch_row (pRes);
	} /* For each result row .. */
	
	/* Check if no error */
	if (mysql_errno (pConn) == 0)
	{
		printf ("%lu rows returned\n\n", (unsigned long) mysql_num_rows (pRes));
	}

	return 0;	
} /* processQueryResult */


/*
 *	Sends query to server
 */
int sendQuery(MYSQL *pConn, char *pCommand)
{
	MYSQL_RES 	*pRes;
	
	if (mysql_query (pConn, pCommand) != 0) /* the query failed */
	{
		/* Better if we have unique error codes for each problem */
		return 1;
	}
	/* the query succeeded; determine whether or not it returns data */
	pRes = mysql_store_result (pConn);
	if (NULL != pRes) /* a result set was returned */
	{
		/* process rows, then free the result set - No error check for now */
		processQueryResult (pConn, pRes);
		mysql_free_result (pRes);
		
	}
	else /* no result set was returned */
	{
		/* Check why no result */
		if (mysql_field_count (pConn) == 0)
		{
			/* Modified table like INSERT command */
			printf ("%lu rows affected\n", (unsigned long) mysql_affected_rows (pConn));
		}
		else /* an error occurred */
		{
			/* Better if we have unique error codes for each problem */
			return 1;
		}
	} /* no result set was returned */
	
	/* Processing OK */
	return 0;
} /* sendQuery */

/* gets input string from the user, and adds ; at the end */
int getSQLInput(char *pBuffer)
{
	int iCount = 0;
	int curData;

	/* Clear excess \n */
	do
	{
		curData = fgetc(stdin);
		*pBuffer = curData;
	} while(*pBuffer == '\n');

	do
	{
		curData = fgetc(stdin);
		pBuffer[++iCount] = curData;

	} while ((curData != '\n') && (curData != EOF));

	if (pBuffer[iCount-1] != ';')
	{
		pBuffer[iCount++]= ';';
		
	}
	pBuffer[iCount]= '\0';

	/* extra line for display */
	fputc('\n', stdout);
	return iCount;
}

/*
 *	Main function
 */
int main(void)
{
	MYSQL 		*pConn;
	char 		aCommand[1024];
	char 		aServer[1024];
	char 		aUser[1024];
	char 		aPassword[1024];
	char 		aDatabase[1024];

	/* Get user inputs */
	printf("Enter Server: ");
	scanf("%s", aServer);

	printf("Enter Database: ");
	scanf("%s", aDatabase);

	printf("Enter user: ");
	scanf("%s", aUser);

	printf("Enter password: ");
	scanf("%s", aPassword);

	/* Initialize MySQL */
	pConn = mysql_init(NULL);

	/* Connect to database */
	if (!mysql_real_connect(pConn, aServer, aUser, aPassword, aDatabase, 0, NULL, 0)) {
		printf("Connect Error: %s\n", mysql_error(pConn));
		return 1;
	}
	
	/* Get queries from the user */
	printf("SQL command > ");
	getSQLInput(aCommand);
	while (strcmp(aCommand, "exit;") != 0)
	{
		if (0 != sendQuery(pConn, aCommand))
		{
			/* Error has occurred during query processing */
			printf("QUERY ERROR: %s\n", mysql_error(pConn));
		}
		printf("SQL command > ");
		getSQLInput(aCommand);
	}

	mysql_close(pConn);
	return 0;
} /* main */
